﻿/*
 * Program Name: New Standards and Operating Rules - Payer Side (POR)
 * Contractor Name: Harris Corporation
 * Contract Number: VA118-11-D-1009
 * Project: POR - CORE Compliance - Automated Eligibility Tool (AET)
 * 
 * Developer: Zach Bertram
 * Date: 01.08.2013
 * ----------------
 * Added documentation for methods and classes
 * Added PING command
 */

using AETShared;
using System;
using System.IO;
using System.Net.Sockets;

namespace AETService
{
    /// <summary>
    /// Command/Control Component Server for sending commands to the AET that aren't in the form of 270 messages
    /// </summary>
    class CCCServer : ServerModel
    {
        const string NL = "\r\n";  //newline
        const int BUFFERSIZE = 4096;  //4 kb
        const string GREETING = "AET Server (v3.0.0.0)";  //ZEB VA118-11-D-1009 1/8/13 updated version number
        const string ERR_ILLEGAL_COMMAND = "-ERR illegal command";
        const string ERR_NO_SUCH_MESSAGE = "-ERR no such message";

        private string username;
        private string password;
        bool Authorized = true;
        //StreamWriter writer = null;

        /// <summary>
        /// Creates a new CCCServer; must call Start() to use the new object
        /// </summary>
        public CCCServer()
        {
        }

        /// <summary>
        /// Creates a new CCCServer that can be used to listen for commands
        /// </summary>
        /// <returns>A brand new instance of CCCServer; must call Start() to use the new object</returns>
        public override ServerModel Clone()
        {
            return new CCCServer();
        }

        /// <summary>
        /// Reads command present on the TCP steam and responds accordingly.
        /// This method is synchronous: it executes on this thread until completed.
        /// </summary>
        /// <param name="tcp">TCP stream to read and write with</param>
        /// <param name="ConfigurationData">Service configuration information</param>
        /// <param name="myConnectedClient"></param>
        public void Start(TcpClient tcp, gDataClass ConfigurationData, ConnectedClient myConnectedClient)
        {
            bool keepAlive = true;
            string response;

            using (StreamReader reader = new StreamReader(tcp.GetStream()))
            {
                using (StreamWriter writer = new StreamWriter(tcp.GetStream()))
                {
                    writer.AutoFlush = true;
                    // Send the greeting
                    writer.WriteLine("+OK " + GREETING);
                    // Get and respond to commands while connected
                    string command = reader.ReadLine();
                    while (keepAlive)
                    {
                        keepAlive = ProcessCommand(command, ConfigurationData, myConnectedClient, out response);
                        writer.WriteLine(response);
                        //Logging.WriteLogFile("CCC Client " + myConnectedClient.remoteEndpointAddress + ";" + myConnectedClient.handle + ";" + myConnectedClient.port + ", sent command: " + command + ", response was: " + response);
                        command = reader.ReadLine();
                    }
                }
            }
            // socket is closed by using statement
        }

        /// <summary>
        /// Performs actions based on the commands read from the TCP stream in Start()
        /// </summary>
        /// <returns>True if more commands should be processed, false if the client has QUIT</returns>
        public bool ProcessCommand(string command, gDataClass ConfigurationData, ConnectedClient myConnectedClient, out string Response)
        {
            // Wait until client sends a vbcrlf
            string[] Modifiers;
            Response = null;
            // Split into parts
            Modifiers = command.Split(' ');
            // First part is the action that the client wants
            // to perform. Subsequent parts are modifiers
            string Action = Modifiers[0].ToUpper().Trim();
            // Decide what to do
            Authorized = CheckAuthorization(username, password);
            if (!Authorized)
            {
                if (Action == "USER" && Modifiers.Length == 2)
                {
                    // The Username was sent, this command should  
                    // have one modifier which is the username
                    username = Modifiers[1];
                    Response = "+OK send password";
                    return true;
                }
                else if (Action == "PASS" && Modifiers.Length == 2)
                {
                    // The Password was sent, this command should have one 
                    // modifier which is the password. The username should  
                    // have been sent already
                    Authorized = CheckAuthorization(username, Modifiers[1]);
                    password = Modifiers[1];
                    if (Authorized)
                    {
                        Response = "+OK user " + username + " logged in";
                    }
                    else
                        Response = "-ERR Authorization failed";
                    return true;
                }
                else if (Action == "PING")  //ZEB VA118-11-D-1009 1/8/13 (+else if) handle new PING command
                {
                    Response = "+OK";
                    return false;  //automatically disconnect after responding to ping
                }
            }
            else
                switch (Action)
                {
                    case "SET_CFG":
                        try
                        {
                            string sCFG = "";
                            if (Modifiers.Length == 2)
                            {
                                Response = sCFG;
                            }
                        }
                        catch (Exception ex)
                        {
                            string s = String.Format("SET_CFG-{0}", ex);
                            //System.Diagnostics.EventLog.WriteEntry("AETService", s, System.Diagnostics.EventLogEntryType.Error);
                        }
                        return true;
                    case "GET_CFG":
                        try
                        {
                            string sCFG = "";
                            Response = sCFG;
                        }
                        catch (Exception ex)
                        {
                            string s = String.Format("REV_G270-{0}", ex);
                            //System.Diagnostics.EventLog.WriteEntry("AETService", s, System.Diagnostics.EventLogEntryType.Error);
                        }
                        return true;
                    case "QUIT":
                        // The client is finished. Delete any marked messages,
                        // then send the "OK" (socket will be closed after 
                        // response is sent)
                        Response = "+OK Goodbye...come back soon!";
                        return false;
                    case "PING":  //ZEB VA118-11-D-1009 1/8/13 (+case) handle new PING command
                        Response = "+OK";
                        return false;  //automatically disconnect after responding to ping
                }
            // if continue not used, it's an illegal command
            Response = ERR_ILLEGAL_COMMAND;
            return true;
        }

        /// <summary>
        /// Skeleton of method for password authenication.
        /// Currently always returns that the password is valid.
        /// </summary>
        /// <param name="Username">The username supplied by the client</param>
        /// <param name="Password">The password supplied by the client</param>
        /// <returns>True</returns>
        public bool CheckAuthorization(string Username, string Password)
        {
            if ((!string.IsNullOrEmpty(Username)) && (!string.IsNullOrEmpty(Password)))
                return true;
            else
                return false;
        }

        /// <summary>
        /// Skeleton of method for decrypting a password to authenticate it against client input.
        /// Currently returns the Password argument unaltered.
        /// </summary>
        /// <param name="Password">The password to be decrypted</param>
        /// <returns>The value of the Password parameter</returns>
        public string DecryptPassword(string Password)
        {
            return Password;
        }

        /// <summary>
        /// Skeleton of method for retrieving encrypted password from storage for a user.
        /// Currently returns an empty string.
        /// </summary>
        /// <param name="Username">The username supplied by the client</param>
        /// <returns>An empty string.</returns>
        public string GetPassword(string Username)
        {
            try
            {

            }
            catch
            {
            }
            return "";
        }
    }
}
