/*
 * Program Name: New Standards and Operating Rules - Payer Side (POR)
 * Contractor Name: Harris Corporation
 * Contract Number: VA118-11-D-1009
 * Project: POR - CORE Compliance - Automated Eligibility Tool (AET)
 * 
 * Developer: Zach Bertram
 * Date: 01.22.2013
 * ----------------
 * Added settings for DB containing Service Type Code info
 */

using System;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using Microsoft.Win32;
using System.Threading;
using System.Runtime.InteropServices;
using System.Collections.Generic;
using AETShared;
using X12Engine;

namespace AETService
{
	public class gDataClass
	{
		[DllImport("kernel32.dll", CharSet = CharSet.Auto)]
		internal static extern void GetSystemInfo([Out] out SYSTEM_INFO systemInfo);

		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
		public struct SYSTEM_INFO
		{
			public InternalSYSTEM_INFO ProcessorInformation;
			public int dwPageSize;
			public int lpMinimumApplicationAddress;
			public int lpMaximumApplicationAddress;
			public int dwActiveProcessorMask;
			public int dwNumberOfProcessors;
			public int dwProcessorType;
			public int dwAllocationGranularity;
			public int dwProcessorLevel;
			public int dwProcessorRevision;
		}
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
		public struct InternalSYSTEM_INFO
		{
			public Architecture wProcessorArchitecture;
			public short wReserved;
		}

		public enum Architecture : ushort
		{
			PROCESSOR_ARCHITECTURE_AMD64 = 9, //Extended 64-bit
			PROCESSOR_ARCHITECTURE_IA64 = 6, //Itanium 64-bit
			PROCESSOR_ARCHITECTURE_INTEL = 0, //32-bit
			PROCESSOR_ARCHITECTURE_UNKNOWN = 0xFFFF //Unknown
		}

		[StructLayout(LayoutKind.Auto, CharSet = CharSet.Auto)]
		private struct ServiceTypeInfo  //ZEB VA118-11-D-1009 1/22/13 (+struct) for storing service type info
		{
			public bool SendGenericResponse;
			public string Deductible;
			public string Copay;
			public string Coinsurance;
		}

		public static readonly object _lock = new object();


		private ReaderWriterLock rwl = new ReaderWriterLock();


		//private int iSection;
		private bool bAllLoadedOK;
		private bool bIs64Bit;
		private bool bTrace;
		private bool bLogPayload;
		private bool bLogWEBService;
		private bool bPreCheck;

		private int iCtlPort;
		private int iAETPort;
		private string sComputerName;

		private string sProcessor;
		private string sOSPlatform;
		private string sOSServicePack;
		private string sOSVersionString;
		private string sOSVersion;


		private string sZipServer;
		private string sZipUser;
		private string sZipPassword;
		private string sZipDatabase = "VAZip";
		//private string sErrorText;

		private string sLEIEServer;
		private string sLEIEUser;
		private string sLEIEPassword;
		private string sLEIEDatabase = "VALEIE";



		private string sANSIServer;
		private string sANSIDatabase;
		private string sANSIUser;
		private string sANSIPassword;

		private string sLogServer;
		private string sLogDatabase;
		private string sLogUser;
		private string sLogPassword;

		private string sSTCServer;  //ZEB VA118-11-D-1009 1/22/13 (+3) added connection information for serv
		private string sSTCDatabase;
		private string sSTCUser;
		private string sSTCPassword;

		private string sSSLName;
		private string sSSLCheck;
		private string sMVIURL;
		private string sMVIPKI;

		// "Keith Baldwin 3/24/2016 "
		public bool EnableLogging { get; set; }
		public bool LogZeroLength { get; set; }
		public string LoggingDirectory { get; set; }
		public string AatServer { get; set; }
		public string AatDatabase { get; set; }
		public string AatUser { get; set; }
		public string AatPassword { get; set; }
		public string CdwServer { get; set; }
		public string CdwDatabase { get; set; }

		private string sPKICheck;

		private string sESURL;
		private string sESUserName;
		private string sESPassword;

		private string sWhiteList;

		private string sZipImportFolder = "";
		private string sLEIEImportFolder = "";
		private string sNPIImportFolder = "";

		private string sZipImportTime;
		private string sLEIEImportTime;
		private string sNPIImportTime;

		private Dictionary<string, ServiceTypeInfo> ServiceTypeCodes = new Dictionary<string, ServiceTypeInfo>();  //ZEB VA118-11-D-1009 1/22/13 (+2) added to store service type info from DB
		private List<string> GenericServiceTypes = new List<string>();
		private String GenericServiceTypeList = "";

		//public delegate void DelegateSendMessage(string to, string message, string[] parameters, System.Net.IPEndPoint ip);
		//public DelegateSendMessage SendMessage;

		public gDataClass()
		{
			//sErrorText = "";

			bAllLoadedOK = false;
			bIs64Bit = false;
			bTrace = false;
			iCtlPort = 0;
			iAETPort = 0;
			sComputerName = "";
			sZipServer = "";
			sZipUser = "";
			sZipPassword = "";
			bAllLoadedOK = Initialize();
		}

		///////////////////////////////////////////////////////////////////////////////////////////////////////////////
		private string ReadRegistry(RegistryKey parentKey, String subKey, String valueName, Object v)
		{
			Utils.LogEvent("ReadRegistry()");
			string sReturnValue = "";
			RegistryKey key;
			try
			{
				key = parentKey.OpenSubKey(subKey, true);
				if (key == null) //If the key doesn't exist.
					throw new Exception("The registry key doesn't exist");
				v = key.GetValue(valueName);      // Get the value.
				sReturnValue = v.ToString();     // Convert the value to a string

				parentKey.Close();            // Close the key.
				key.Close();
			}
			catch (Exception ex)
			{
				string errText = ex.ToString();
				string errMsg = ex.Message;
				string errSource = ex.Source;
				//System.Diagnostics.EventLog.WriteEntry("AETService", errText, System.Diagnostics.EventLogEntryType.Error, 300, 500);
			}
			return sReturnValue;
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		private bool Initialize()
		{
			bool bReturn = false;
			Object v = null;
			if (ProcessorAndOSInformation())
			{
				if (Is64Bit)
				{
				}
				sComputerName = ReadRegistry(Registry.LocalMachine, "SYSTEM\\CurrentControlSet\\Control\\ComputerName\\ComputerName", "ComputerName", v);
				bReturn = GetServiceParameters();
			}
			else
			{
				// Error Getting Processor or OS information
			}
			return bReturn;
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		private bool ProcessorAndOSInformation()
		{
			bool bReturn = false;
			SYSTEM_INFO systemInfo;
			GetSystemInfo(out systemInfo);
			switch (systemInfo.ProcessorInformation.wProcessorArchitecture)
			{
				case Architecture.PROCESSOR_ARCHITECTURE_INTEL:
					sProcessor = "32-bit Processor";
					Is64Bit = false;
					bReturn = true;
					break;
				case Architecture.PROCESSOR_ARCHITECTURE_IA64:
					sProcessor = "64-bit Itanium Processor";
					Is64Bit = true;
					bReturn = true;
					break;
				case Architecture.PROCESSOR_ARCHITECTURE_AMD64:
					sProcessor = "64-bit extended Processor.";
					Is64Bit = true;
					bReturn = true;
					break;
				case Architecture.PROCESSOR_ARCHITECTURE_UNKNOWN:
					sProcessor = "UNKNOWN Processor";
					Is64Bit = false;
					bReturn = false;
					break;
				default:
					sProcessor = "UNKNOWN Processor";
					Is64Bit = false;
					bReturn = false;
					break;
			}

			OperatingSystem os = Environment.OSVersion;
			sOSPlatform = os.Platform.ToString();
			sOSServicePack = os.ServicePack;
			sOSVersionString = os.VersionString;
			sOSVersion = os.Version.ToString();

			//string x86 = System.Environment.GetEnvironmentVariable("ProgramFiles(x86)");
			//string x = System.Environment.GetEnvironmentVariable("ProgramFiles");

			return bReturn;
		}

		public bool SetServiceParameters()
		{
			bool bReturn = false;

			Microsoft.Win32.RegistryKey system,
				 currentControlSet,
				 services,
				 service,
				 config;
			try
			{
				string user = Environment.UserDomainName + "\\" + Environment.UserName;

				///////////////////////////////////////////////////////////////////////////////////////////////////////////////
				// Open the HKEY_LOCAL_MACHINE\SYSTEM key
				// HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Service\AlphaIIDispatch\Parameters
				///////////////////////////////////////////////////////////////////////////////////////////////////////////////
				system = Microsoft.Win32.Registry.LocalMachine.OpenSubKey("System");
				//Open CurrentControlSet
				currentControlSet = system.OpenSubKey("CurrentControlSet");
				//Go to the services key
				services = currentControlSet.OpenSubKey("Services");
				//Open the key for your service, and allow writing
				service = services.OpenSubKey("AETService", true);
				//Add your service's description as a REG_SZ value named "Description"
				//(Optional) Add some custom information your service will use...
				config = service.OpenSubKey("Parameters", true);

				string s1 = String.Format("Zip={0};LEIE={1};NPI={2}", sZipImportTime, sLEIEImportTime, sNPIImportTime);
				config.SetValue("ImportUpdate", s1);

				bReturn = true;

				config.Close();
				service.Close();
				services.Close();
				currentControlSet.Close();
				system.Close();
				///////////////////////////////////////////////////////////////////////////////////////////////////////////////
			}
			catch (Exception ex)
			{
				//System.IO.StreamWriter writer = new System.IO.StreamWriter(@"LogFile.txt", true);
				//writer.WriteLine(ex.Message);

				//writer.Close();
				// Can not write ro reg. Premisions
				Logging.WriteLogFile(ex.Message + "\r\n" + ex.StackTrace);
			}

			return bReturn;
		}

		private bool GetServiceParameters()
		{
			Utils.LogEvent("GetServiceParameters()");

			bool bReturn = false;
			int iSection = 0;

			Microsoft.Win32.RegistryKey system,
				 currentControlSet,
				 services,
				 service,
				 config;
			try
			{
				///////////////////////////////////////////////////////////////////////////////////////////////////////////////
				// Open the HKEY_LOCAL_MACHINE\SYSTEM key
				// HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Service\AlphaIIDispatch\Parameters
				///////////////////////////////////////////////////////////////////////////////////////////////////////////////
				iSection = 1;
				system = Microsoft.Win32.Registry.LocalMachine.OpenSubKey("System");
				//Open CurrentControlSet
				currentControlSet = system.OpenSubKey("CurrentControlSet");
				//Go to the services key
				services = currentControlSet.OpenSubKey("Services");
				//Open the key for your service, and allow writing
				service = services.OpenSubKey("AETService", true);
				//Add your service's description as a REG_SZ value named "Description"
				//(Optional) Add some custom information your service will use...
				config = service.OpenSubKey("Parameters");
				///////////////////////////////////////////////////////////////////////////////////////////////////////////////

				///////////////////////////////////////////////////////////////////////////////////////////////////////////////
				// Start extracting the appropriate data from there keys.

				// Get the control port (iCtlPort) from the Registry CtlPort
				try
				{
					string sUpd;
					string[] UpText;

					sUpd = config.GetValue("ImportUpdate").ToString();
					//AETShared.Logging.WriteLogFile("ImportUpdate from Registry = " + sUpd);
					//////////////////////////////////////////////////////////////////////////////////

					try
					{
						sZipImportTime = string.Empty;
						sLEIEImportTime = string.Empty;
						sNPIImportTime = string.Empty;
						if (!string.IsNullOrWhiteSpace(sUpd))
						{
							UpText = sUpd.Split(';');
							if ((UpText.Length >= 1) && (UpText[0].Contains("Zip=")))
								sZipImportTime = UpText[0].Replace("Zip=", "");

							if ((UpText.Length >= 1) && (UpText[1].Contains("LEIE=")))
								sLEIEImportTime = UpText[1].Replace("LEIE=", "");

							if ((UpText.Length >= 1) && (UpText[2].Contains("NPI=")))
								sNPIImportTime = UpText[2].Replace("NPI=", "");
						}
					}
					catch (Exception ex)
					{
						//System.IO.StreamWriter writer = new System.IO.StreamWriter(@"LogFile.txt", true);
						//writer.WriteLine(ex.Message);
						//writer.Close();
						Logging.WriteLogFile(ex.Message + "\r\n" + ex.StackTrace);
					}

					//ZEB VA118-11-D-1009 2/7/13 (+try/catch) fixed AETPort/CtlPort reversal
					try
					{
						iAETPort = System.Int32.Parse(Crypto.AES.Decrypt(config.GetValue("AETPort").ToString()));
					}
					catch
					{
						iAETPort = System.Int32.Parse(config.GetValue("AETPort").ToString());
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:CtlPort Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 302, 500);
				}
				// Get the clearing house port (iCWCSPort) from the Registry CWCSPort 
				try
				{
					iSection = 3;

					//ZEB VA118-11-D-1009 2/7/13 (+try/catch) fixed AETPort/CtlPort reversal
					try
					{
						iCtlPort = System.Int32.Parse(Crypto.AES.Decrypt(config.GetValue("CtlPort").ToString()));
					}
					catch
					{

						iCtlPort = System.Int32.Parse(config.GetValue("CtlPort").ToString());
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:AETPort Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 304, 500);
				}
				///////////////////////////////////////////////////////////////////////////////////////////////////////////////
				// Get ZipCode Database Server
				try
				{
					iSection = 4;
					//Steve Speck 05/15/2011 - Added decryption of database config values
					try
					{
						sZipServer = Crypto.AES.Decrypt(config.GetValue("ZipServer").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of zip server failed: " + ex.Message;
						////System.Diagnostics.EventLog.WriteEntry("ZipServer", errMsg, System.Diagnostics.EventLogEntryType.Error,306,500);
						sZipServer = config.GetValue("ZipServer").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:ZipServer Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 308, 500);
				}

				///////////////////////////////////////////////////////////////////////////////////////////////////////////////
				// Get ZipCode Database 
				try
				{
					iSection = 4;
					//Steve Speck 05/15/2011 - Added decryption of database config values
					try
					{
						sZipDatabase = Crypto.AES.Decrypt(config.GetValue("ZipDatabase").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of zip database failed: " + ex.Message;
						////System.Diagnostics.EventLog.WriteEntry("ZipDatabase", errMsg, System.Diagnostics.EventLogEntryType.Error,310,500);
						sZipDatabase = config.GetValue("ZipDatabase").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:ZipDatabase Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 312, 500);
				}

				// Get ZipCode Database User
				try
				{
					iSection = 5;
					//Steve Speck 05/15/2011 - Added decryption of database configuration
					try
					{
						sZipUser = Crypto.AES.Decrypt(config.GetValue("ZipUser").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of zip user failed: " + ex.Message;
						////System.Diagnostics.EventLog.WriteEntry("ZipUser", errMsg, System.Diagnostics.EventLogEntryType.Error,314,500);
						sZipUser = config.GetValue("ZipUser").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:ZipUser Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 316, 500);
				}
				// Get ZipCode Database Passsword
				try
				{
					iSection = 6;
					//Steve Speck 05/15/2011 - Added decryption of database configuration
					try
					{
						sZipPassword = Crypto.AES.Decrypt(config.GetValue("ZipPassword").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of zip password failed: " + ex.Message;
						////System.Diagnostics.EventLog.WriteEntry("ZipPassword", errMsg, System.Diagnostics.EventLogEntryType.Error,318,500);
						sZipPassword = config.GetValue("ZipPassword").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:ZipPassword Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 320, 500);
				}

				///////////////////////////////////////////////////////////////////////////////////////////////////////////////
				// Get LEIECode Database Server
				try
				{
					iSection = 4;
					//Steve Speck 05/15/2011 - Added decryption of database config values
					try
					{
						sLEIEServer = Crypto.AES.Decrypt(config.GetValue("LEIEServer").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of zip server failed: " + ex.Message;
						////System.Diagnostics.EventLog.WriteEntry("LEIEServer", errMsg, System.Diagnostics.EventLogEntryType.Error, 306, 500);
						sLEIEServer = config.GetValue("LEIEServer").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:LEIEServer Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 308, 500);
				}

				///////////////////////////////////////////////////////////////////////////////////////////////////////////////
				// Get LEIECode Database 
				try
				{
					iSection = 4;
					//Steve Speck 05/15/2011 - Added decryption of database config values
					try
					{
						sLEIEDatabase = Crypto.AES.Decrypt(config.GetValue("LEIEDatabase").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of zip database failed: " + ex.Message;
						////System.Diagnostics.EventLog.WriteEntry("LEIEDatabase", errMsg, System.Diagnostics.EventLogEntryType.Error, 310, 500);
						sLEIEDatabase = config.GetValue("LEIEDatabase").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:LEIEDatabase Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 312, 500);
				}

				// Get LEIECode Database User
				try
				{
					iSection = 5;
					//Steve Speck 05/15/2011 - Added decryption of database configuration
					try
					{
						sLEIEUser = Crypto.AES.Decrypt(config.GetValue("LEIEUser").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of zip user failed: " + ex.Message;
						////System.Diagnostics.EventLog.WriteEntry("LEIEUser", errMsg, System.Diagnostics.EventLogEntryType.Error, 314, 500);
						sLEIEUser = config.GetValue("LEIEUser").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:LEIEUser Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 316, 500);
				}
				// Get LEIECode Database Passsword
				try
				{
					iSection = 6;
					//Steve Speck 05/15/2011 - Added decryption of database configuration
					try
					{
						sLEIEPassword = Crypto.AES.Decrypt(config.GetValue("LEIEPassword").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of zip password failed: " + ex.Message;
						////System.Diagnostics.EventLog.WriteEntry("LEIEPassword", errMsg, System.Diagnostics.EventLogEntryType.Error, 318, 500);
						sLEIEPassword = config.GetValue("LEIEPassword").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:LEIEPassword Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 320, 500);
				}

				///////////////////////////////////////////////////////////////////////////////////////////////////////////////
				// Get Ansi Database Server
				try
				{
					iSection = 7;
					// Steve Speck 05/15/2011 - Added decryption of database configuration
					try
					{
						sANSIServer = Crypto.AES.Decrypt(config.GetValue("ANSIServer").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of ANSI server failed: " + ex.Message;
						////System.Diagnostics.EventLog.WriteEntry("ANSIServer", errMsg, System.Diagnostics.EventLogEntryType.Error,322,500);
						sANSIServer = config.GetValue("ANSIServer").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:ANSIServer Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 324, 500);
				}
				// Get ZipCode Database
				try
				{
					iSection = 8;
					// Steve Speck 05/15/2011 - Added decryption of database configuration
					try
					{
						sANSIDatabase = Crypto.AES.Decrypt(config.GetValue("ANSIDatabase").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of zip server failed: " + ex.Message;
						////System.Diagnostics.EventLog.WriteEntry("ANSIDatabase", errMsg, System.Diagnostics.EventLogEntryType.Error,326,500);
						sANSIDatabase = config.GetValue("ANSIDatabase").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:ANSIDatabase Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 328, 500);
				}
				// Get ZipCode Database User
				try
				{
					iSection = 9;
					//Steve Speck 05/15/2011 - Added decryption of database configuration
					try
					{
						sANSIUser = Crypto.AES.Decrypt(config.GetValue("ANSIUser").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of zip server failed: " + ex.Message;
						////System.Diagnostics.EventLog.WriteEntry("ANSIUser", errMsg, System.Diagnostics.EventLogEntryType.Error,330,500);

						sANSIUser = config.GetValue("ANSIUser").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:ANSIUser Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 332, 500);
				}
				// Get ZipCode Database Passsword
				try
				{
					iSection = 10;
					//Steve Speck 05/15/2011 Added decryption of database configuration
					try
					{
						sANSIPassword = Crypto.AES.Decrypt(config.GetValue("ANSIPassword").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of ANSI password failed: " + ex.Message;
						////System.Diagnostics.EventLog.WriteEntry("ANSIUser", errMsg, System.Diagnostics.EventLogEntryType.Error,334,500);
						sANSIPassword = config.GetValue("ANSIPassword").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:ANSIPassword Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 336, 500);
				}

				/////////////////////////////////////////////////
				// get the database info for the audit log
				// 
				try
				{
					iSection = 11;
					//Steve Speck 05/15/2011 - Added decryption of database configuration
					try
					{
						sLogServer = Crypto.AES.Decrypt(config.GetValue("LogServer").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of Log server failed: " + ex.Message;
						////System.Diagnostics.EventLog.WriteEntry("LogServer", errMsg, System.Diagnostics.EventLogEntryType.Error,338,500);
						sLogServer = config.GetValue("LogServer").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:LogServer Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 340, 500);
				}
				// Get Audit Database
				try
				{
					iSection = 12;
					//Steve Speck 05/15/2011 - Added decryption of database configuration
					try
					{
						sLogDatabase = Crypto.AES.Decrypt(config.GetValue("LogDatabase").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of LogDatabase failed: " + ex.Message;
						////System.Diagnostics.EventLog.WriteEntry("LogDatabase", errMsg, System.Diagnostics.EventLogEntryType.Error,342,500);
						sLogDatabase = config.GetValue("LogDatabase").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:LogDatabase Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 344, 500);
				}
				// Get Audit Database User
				try
				{
					iSection = 13;
					//Steve Speck 05/15/2011 - Added decryption of database configuration
					try
					{
						sLogUser = Crypto.AES.Decrypt(config.GetValue("LogUser").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of LogUser failed: " + ex.Message;
						////System.Diagnostics.EventLog.WriteEntry("LogUser", errMsg, System.Diagnostics.EventLogEntryType.Error,346,500);
						sLogUser = config.GetValue("LogUser").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:LogUser Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 348, 500);
				}
				// Get Audit Database Passsword
				try
				{
					iSection = 14;
					//Steve Speck 05/15/2011 - Add decryption of database configruation
					try
					{
						sLogPassword = Crypto.AES.Decrypt(config.GetValue("LogPassword").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of LogPassword failed: " + ex.Message;
						////System.Diagnostics.EventLog.WriteEntry("LogPassword", errMsg, System.Diagnostics.EventLogEntryType.Error,350,500);
						sLogPassword = config.GetValue("LogPassword").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:LogPassword Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 352, 500);
				}

				//ZEB VA118-11-D-1009 1/22/13 (+80) added to get service type connection info
				/////////////////////////////////////////////////
				// get the database info for the service type data
				// 
				try
				{
					iSection = 15;
					try
					{
						sSTCServer = Crypto.AES.Decrypt(config.GetValue("STCServer").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of STC server failed: " + ex.Message;
						sSTCServer = config.GetValue("STCServer").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:STCServer Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 353, 500);
				}
				// Get STC Database
				try
				{
					iSection = 16;
					try
					{
						sSTCDatabase = Crypto.AES.Decrypt(config.GetValue("STCDatabase").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of STCDatabase failed: " + ex.Message;
						sSTCDatabase = config.GetValue("STCDatabase").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:STCDatabase Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 354, 500);
				}
				// Get STC Database User
				try
				{
					iSection = 17;
					try
					{
						sSTCUser = Crypto.AES.Decrypt(config.GetValue("STCUser").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of STCUser failed: " + ex.Message;
						sSTCUser = config.GetValue("STCUser").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:STCUser Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 355, 500);
				}
				// Get STC Database Passsword
				try
				{
					iSection = 18;
					//Steve Speck 05/15/2011 - Add decryption of database configruation
					try
					{
						sSTCPassword = Crypto.AES.Decrypt(config.GetValue("STCPassword").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of STCPassword failed: " + ex.Message;
						sSTCPassword = config.GetValue("STCPassword").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:STCPassword Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 356, 500);
				}

				#region "Keith Baldwin 3/24/2016 "

				try
				{
					iSection = 15;
					bool tempBool = false;
					try
					{
						bool.TryParse(config.GetValue("EnableLogging").ToString(), out tempBool);
						EnableLogging = tempBool;
					}
					catch (Exception ex)
					{
						string errMsg = "Parsing of EnableLogging failed: " + ex.Message;
						EnableLogging = false;
					}
					try
					{
						bool.TryParse(config.GetValue("LogZeroLength").ToString(), out tempBool);
						LogZeroLength = tempBool;
					}
					catch (Exception ex)
					{
						string errMsg = "Parsing of LogZeroLength failed: " + ex.Message;
						LogZeroLength = false;
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:Parsing EnableLogging Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 353, 500);
				}

				try
				{
					iSection = 15;
					try
					{
						LoggingDirectory = Crypto.AES.Decrypt(config.GetValue("LoggingDirectory").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of LoggingDirectory failed: " + ex.Message;
						LoggingDirectory = config.GetValue("LoggingDirectory").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:AatServer Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 353, 500);
				}

				try
				{
					iSection = 15;
					try
					{
						AatServer = Crypto.AES.Decrypt(config.GetValue("AatServer").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of AAT server failed: " + ex.Message;
						AatServer = config.GetValue("AatServer").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:AatServer Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 353, 500);
				}

				try
				{
					iSection = 16;
					try
					{
						AatDatabase = Crypto.AES.Decrypt(config.GetValue("AatDatabase").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of AatDatabase failed: " + ex.Message;
						AatDatabase = config.GetValue("AatDatabase").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:AatDatabase Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 354, 500);
				}

				try
				{
					iSection = 17;
					try
					{
						AatUser = Crypto.AES.Decrypt(config.GetValue("AatDatabaseUsername").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of AatDatabaseUsername failed: " + ex.Message;
						AatUser = config.GetValue("AatDatabaseUsername").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:AatDatabaseUsername Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 355, 500);
				}

				try
				{
					iSection = 18;
					//Steve Speck 05/15/2011 - Add decryption of database configruation
					try
					{
						AatPassword = Crypto.AES.Decrypt(config.GetValue("AatDatabasePassword").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of AatDatabasePassword failed: " + ex.Message;
						AatPassword = config.GetValue("AatDatabasePassword").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:AatDatabasePassword Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 356, 500);
				}

				try
				{
					iSection = 15;
					try
					{
						CdwServer = Crypto.AES.Decrypt(config.GetValue("CdwServer").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of Cdw server failed: " + ex.Message;
						CdwServer = config.GetValue("CdwServer").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:CdwServer Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 353, 500);
				}

				try
				{
					iSection = 16;
					try
					{
						CdwDatabase = Crypto.AES.Decrypt(config.GetValue("CdwDatabase").ToString());
					}
					catch (Exception ex)
					{
						string errMsg = "Decrytpion of CdwDatabase failed: " + ex.Message;
						CdwDatabase = config.GetValue("CdwDatabase").ToString();
					}
				}
				catch (Exception e)
				{
					string errMsg = String.Format("REG:CdwDatabase Failed {0}", e.Message);
					//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 354, 500);
				}


				#endregion


				try
				{
					sSSLCheck = Crypto.AES.Decrypt(config.GetValue("SSLNameVC").ToString());
				}
				catch
				{
					sSSLCheck = config.GetValue("SSLNameVC").ToString();
				}

				try
				{
					sMVIURL = Crypto.AES.Decrypt(config.GetValue("MVIURL").ToString());
				}
				catch
				{
					sMVIURL = config.GetValue("MVIURL").ToString();
				}

				try
				{
					sMVIPKI = Crypto.AES.Decrypt(config.GetValue("MVIPKI").ToString());
				}
				catch
				{
					sMVIPKI = config.GetValue("MVIPKI").ToString();
				}

				try
				{
					sMVIPKI = Crypto.AES.Decrypt(config.GetValue("MVIPKI").ToString());
				}
				catch
				{
					sMVIPKI = config.GetValue("MVIPKI").ToString();
				}

				try
				{
					sPKICheck = Crypto.AES.Decrypt(config.GetValue("MVIPKIVC").ToString());
				}
				catch
				{
					sPKICheck = config.GetValue("MVIPKIVC").ToString();
				}

				try
				{
					sESURL = Crypto.AES.Decrypt(config.GetValue("ESURL").ToString());
				}
				catch
				{
					sESURL = config.GetValue("ESURL").ToString();
				}

				try
				{
					sESUserName = Crypto.AES.Decrypt(config.GetValue("ESUN").ToString());
				}
				catch
				{
					sESUserName = config.GetValue("ESUN").ToString();
				}

				try
				{
					sESPassword = Crypto.AES.Decrypt(config.GetValue("ESPW").ToString());
				}
				catch
				{
					sESPassword = config.GetValue("ESPW").ToString();
				}

				try
				{
					sSSLName = Crypto.AES.Decrypt(config.GetValue("SSLName").ToString());
				}
				catch
				{
					sSSLName = config.GetValue("SSLName").ToString();
				}

				try
				{
					sWhiteList = Crypto.AES.Decrypt(config.GetValue("IPWL").ToString());
				}
				catch
				{
					sWhiteList = config.GetValue("IPWL").ToString();
				}

				try
				{
					sZipImportFolder = Crypto.AES.Decrypt(config.GetValue("ZipImportFolder").ToString());
				}
				catch
				{
					sZipImportFolder = config.GetValue("ZipImportFolder").ToString();
				}

				try
				{
					sLEIEImportFolder = Crypto.AES.Decrypt(config.GetValue("LEIEImportFolder").ToString());
				}
				catch
				{
					sLEIEImportFolder = config.GetValue("LEIEImportFolder").ToString();
				}

				try
				{
					sNPIImportFolder = Crypto.AES.Decrypt(config.GetValue("NPIImportFolder").ToString());
				}
				catch
				{
					sNPIImportFolder = "";
				}
				///////////////////////////////////////////////////////////////////////////////////////////////////////////////
				// We are done extracting out key data,
				// Close out and clean up the Registry objects
				iSection = 19;

				config.Close();
				service.Close();
				services.Close();
				currentControlSet.Close();
				system.Close();
				bReturn = true;
			}
			catch (Exception ex)
			{
				string errMsg = String.Format("Registry Extraction Error: {0} {1}", iSection, ex);
				//System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 354, 500);
				Logging.WriteLogFile(errMsg + "\r\n" + ex.StackTrace);
			}
			return bReturn;
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public void AuditLog(int LogID, string AuditData, System.Diagnostics.EventLogEntryType Mode, int evnt, short cag)   // the InstallFolder property
		{
			//Acquire a read lock on the resource.
			rwl.AcquireReaderLock(Timeout.Infinite);

			try
			{
				string msg = String.Format("[{0}]-{1}", LogID, AuditData);
				//System.Diagnostics.EventLog.WriteEntry("AETService", msg, Mode, evnt, cag);
			}
			finally
			{
				//Release the lock.
				rwl.ReleaseReaderLock();
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public void AuditLog(int LogID, string AuditData, System.Diagnostics.EventLogEntryType Mode)   // the InstallFolder property
		{
			//Acquire a read lock on the resource.
			rwl.AcquireReaderLock(Timeout.Infinite);

			try
			{
				string msg = String.Format("[{0}]-{1}", LogID, AuditData);
				//System.Diagnostics.EventLog.WriteEntry("AETService", msg, Mode);
			}
			finally
			{
				//Release the lock.
				rwl.ReleaseReaderLock();
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public void AuditLog(string AuditData, System.Diagnostics.EventLogEntryType Mode)   // the InstallFolder property
		{
			//Acquire a read lock on the resource.
			rwl.AcquireReaderLock(Timeout.Infinite);

			try
			{
				//System.Diagnostics.EventLog.WriteEntry("AETService", AuditData, Mode);
			}
			finally
			{
				//Release the lock.
				rwl.ReleaseReaderLock();
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string GetProcessor()   // the InstallFolder property
		{
			string sMsg = "";
			//Acquire a read lock on the resource.
			rwl.AcquireReaderLock(Timeout.Infinite);

			try
			{
				sMsg = String.Format("OSVersionString:{0}\r\nProcessor:{1}", sOSVersionString, sProcessor);
			}
			finally
			{
				//Release the lock.
				rwl.ReleaseReaderLock();
			}
			return sMsg;
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public bool Is64Bit   // the ControlPort property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return bIs64Bit;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					bIs64Bit = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}
		public bool TraceEnabled   // the ControlPort property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return bTrace;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					bTrace = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public bool LogPayload
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return bLogPayload;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					bLogPayload = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public bool LogWEBService
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return bLogWEBService;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					bLogWEBService = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public bool PreCheck
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return bPreCheck;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					bPreCheck = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public int CtlPort   // the CtlPort property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return iCtlPort;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					iCtlPort = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public int AETPort   // the RouterPort property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return iAETPort;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					iAETPort = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string ZipServer   // the ZipCode Database server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sZipServer;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sZipServer = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string ZipDatabase   // the ZipCode Database server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sZipDatabase;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sZipDatabase = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string ZipUser   // the ZipCode Database User property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sZipUser;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sZipUser = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string ZipPassword   // the ZipCode Database User property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sZipPassword;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sZipPassword = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}


		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string LEIEServer   // the LEIECode Database server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sLEIEServer;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sLEIEServer = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string LEIEDatabase   // the LEIECode Database server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sLEIEDatabase;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sLEIEDatabase = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string LEIEUser   // the LEIECode Database User property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sLEIEUser;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sLEIEUser = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string LEIEPassword   // the LEIECode Database User property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sLEIEPassword;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sLEIEPassword = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}


		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string AnsiServer   // the Ansi Database server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sANSIServer;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sANSIServer = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		public string AnsiDatabase   // the Ansi Database property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sANSIDatabase;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sANSIDatabase = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string AnsiUser   // the Ansi Database User property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sANSIUser;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sANSIUser = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string AnsiPassword   // the Ansi Database password property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sANSIPassword;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sANSIPassword = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string LogServer   // the Log Database server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sLogServer;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sLogServer = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		public string LogDatabase   // the Log Database property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sLogDatabase;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sLogDatabase = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string LogUser   // the Log Database User property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sLogUser;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sLogUser = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string LogPassword   // the Log Database password property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sLogPassword;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sLogPassword = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string GetComputerName() // the computerName property
		{
			rwl.AcquireReaderLock(Timeout.Infinite);
			try
			{
			}
			finally
			{
				//Release the lock.
				rwl.ReleaseReaderLock();
			}
			return sComputerName;
		}
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public bool AllLoadedOK   // the AllLoadedOK property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return bAllLoadedOK;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					bAllLoadedOK = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string SSLCheck   // the SSLCheck server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sSSLCheck;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sSSLCheck = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string PKICheck   // the PKICheck server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sPKICheck;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sPKICheck = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}



		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string SSLName   // the SSLName server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sSSLName;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sSSLName = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string MVIPKI   // the sMVIPKI server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sMVIPKI;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sMVIPKI = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}

		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string WhiteList   // the WhiteList server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sWhiteList;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sWhiteList = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string ZipImportTime   // the WhiteList server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sZipImportTime;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sZipImportTime = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string ZipImportFolder   // the WhiteList server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sZipImportFolder;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sZipImportFolder = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string LEIEImportTime   // the WhiteList server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sLEIEImportTime;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sLEIEImportTime = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string LEIEImportFolder   // the WhiteList server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sLEIEImportFolder;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sLEIEImportFolder = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}


		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string NPIImportTime   // the WhiteList server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sNPIImportTime;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sNPIImportTime = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string NPIImportFolder   // the WhiteList server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sNPIImportFolder;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sNPIImportFolder = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}



		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string MVIURL   // the MVIURL server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sMVIURL;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sMVIURL = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string ESURL   // the ESURL server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sESURL;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sESURL = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string ESUserName   // the ESUserName server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sESUserName;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sESUserName = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		public string ESPassword   // the sESPassword server property
		{
			get
			{
				//Acquire a read lock on the resource.
				rwl.AcquireReaderLock(Timeout.Infinite);
				try
				{
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseReaderLock();
				}
				return sESPassword;
			}
			set
			{
				//Acquire a write lock on the resource.
				rwl.AcquireWriterLock(Timeout.Infinite);
				try
				{
					sESPassword = value;
				}
				finally
				{
					//Release the lock.
					rwl.ReleaseWriterLock();
				}
			}
		}

		public int LogRequest(string ClearingHouseCtlID, string payload, string IPAddress)
		{
			int rtn = -1;

			rwl.AcquireWriterLock(Timeout.Infinite);

			string SQL = "usp_LogRequest";
			SqlConnectionStringBuilder scb = new SqlConnectionStringBuilder
			{
				PersistSecurityInfo = false,
				IntegratedSecurity = false,
				DataSource = sLogServer,
				InitialCatalog = sLogDatabase,
				UserID = sLogUser,
				Password = sLogPassword
			};

			SqlCommand cmd = new SqlCommand(SQL, new SqlConnection(scb.ConnectionString))
			{
				CommandType = CommandType.StoredProcedure
			};
			try
			{
				cmd.Parameters.Add("@ClearingHouseCtlID", SqlDbType.NChar);
				cmd.Parameters["@ClearingHouseCtlID"].Value = ClearingHouseCtlID;


				cmd.Parameters.AddWithValue("@RequestTime", DateTime.Now);

				cmd.Parameters.Add("@InboundPayload", SqlDbType.NVarChar);
				//cmd.Parameters["@InboundPayload"].Size = payload.Length * 2;
				cmd.Parameters["@InboundPayload"].Value = payload;

				cmd.Parameters.Add("@ResponseIPAddress", SqlDbType.NChar);
				cmd.Parameters["@ResponseIPAddress"].Value = IPAddress;

				cmd.Connection.Open();
				object tmprtn = cmd.ExecuteScalar();
				rtn = int.Parse(tmprtn.ToString());
			}
			catch (Exception ex)
			{
				string erMsg = String.Format("Log Request failed: {0}\r\nServer: {1}, Database: {2}, UserID: {3}, UserData: {4}", ex.Message, sLogServer, sLogDatabase, sLogUser, sLogPassword);
				AuditLog(erMsg, System.Diagnostics.EventLogEntryType.Error);
			}
			finally
			{
				if (cmd.Connection != null)
				{
					if (cmd.Connection.State != ConnectionState.Closed)
					{
						cmd.Connection.Close();
					}
				}
				cmd.Dispose();
				cmd = null;
				rwl.ReleaseWriterLock();
			}

			return rtn;
		}

		public string GetEDIData(int key)
		{
			string rtn = string.Empty;

			rwl.AcquireWriterLock(Timeout.Infinite);
			string selectStatment = "SELECT [ClearingHouseCtlID] " +
			",[RequestTime] " +
			",[RequestQualifier] " +
			",[ResponseTime] " +
			",[ResponseQualifier] " +
			",[StatusIndicator] " +
			",[ErrorID] " +
			",[ErrorMessage] " +
			",[InboundPayload] " +
			",[OutboundPayload] " +
			",[ResponseIPAddress] " +
			"FROM [VA_Audit].[dbo].[BenefitAuditLog] where [k_pk_id] ";

			SqlCommand cmd = null;
			SqlConnection connectionObj = null;
			try
			{
				string SQL = String.Format("{0}={1}", selectStatment, key);
				//SqlConnectionStringBuilder scb = new SqlConnectionStringBuilder();
				//scb.PersistSecurityInfo = false;
				//scb.IntegratedSecurity = false;
				//scb.DataSource = sLogServer;
				//scb.InitialCatalog = sLogDatabase;
				//scb.UserID = sLogUser;
				//scb.Password = sLogPassword;

				string sCWCSConnectionString = String.Format("Data Source={0};Initial Catalog={1};User ID={2};Password={3};Timeout=30;", sLogServer, sLogDatabase, sLogUser, sLogPassword);
				connectionObj = new SqlConnection(sCWCSConnectionString);
				connectionObj.Open();
				cmd = new SqlCommand(SQL, connectionObj);

				SqlDataReader reader = cmd.ExecuteReader();
				if (reader.HasRows)
				{
					reader.Read();
					rtn = String.Format("\r\nKey={0}\r\n{1}-------------------\r\n{2}\r\n", key, reader.GetValue(8), reader.GetValue(9));

				}
				else
				{

					rtn = String.Format("No EDI data returned for key: {0}", key);


				}
				reader.Close();
				reader.Dispose();
			}
			catch (Exception ex)
			{
				AuditLog("GetEDIData: " + ex.Message, System.Diagnostics.EventLogEntryType.Error);
				rtn = "ERROR: " + ex.Message;
			}
			finally
			{
				if (cmd.Connection != null)
				{
					if (cmd.Connection.State != ConnectionState.Closed)
					{
						cmd.Connection.Close();
					}
				}
				cmd.Dispose();
				cmd = null;
				connectionObj.Close();
				connectionObj.Dispose();
				connectionObj = null;
				rwl.ReleaseWriterLock();
			}

			return rtn;
		}

		public int LogError(int LogID, int ErrorID, string ErrorMsg)
		{


			int rtn = -1;

			rwl.AcquireWriterLock(Timeout.Infinite);
			string sCWCSConnectionString1 = String.Format("Data Source={0};Initial Catalog={1};User ID={2};Password={3};Timeout=30;", sLogServer, sLogDatabase, sLogUser, sLogPassword);
			SqlConnection connectionObj1 = new SqlConnection(sCWCSConnectionString1);
			connectionObj1.Open();

			// 
			string sQuery1 = String.Format("UPDATE BenefitAuditLog SET ErrorId={0}, ErrorMessage='{1}' WHERE k_pk_id = {2}", ErrorID, ErrorMsg, LogID);
			SqlCommand QueryCommand1 = new SqlCommand(sQuery1, connectionObj1);
			SqlDataReader readerA1 = QueryCommand1.ExecuteReader();

			if (readerA1.HasRows)
			{

			}
			else
			{
				////System.Diagnostics.EventLog.WriteEntry("AETService", sQuery, System.Diagnostics.EventLogEntryType.Error);
			}
			readerA1.Close();
			QueryCommand1.Dispose();
			connectionObj1.Close();
			rwl.ReleaseWriterLock();

			return rtn;
		}


		public int LogMVI(int LogID, float runtime, string status, string payload)
		{


			int rtn = -1;

			rwl.AcquireWriterLock(Timeout.Infinite);

			string SQL = "usp_LogMVI";
			SqlConnectionStringBuilder scb = new SqlConnectionStringBuilder();
			scb.PersistSecurityInfo = false;
			scb.IntegratedSecurity = false;
			scb.DataSource = sLogServer;
			scb.InitialCatalog = sLogDatabase;
			scb.UserID = sLogUser;
			scb.Password = sLogPassword;

			SqlCommand cmd = new SqlCommand(SQL, new SqlConnection(scb.ConnectionString));
			cmd.CommandType = CommandType.StoredProcedure;
			try
			{

				cmd.Parameters.AddWithValue("@k_pk_id", LogID);
				cmd.Parameters.AddWithValue("@MVITime", runtime);
				cmd.Parameters.AddWithValue("@MVIStatus", status);
				cmd.Parameters.AddWithValue("@MVIPayloadData", payload);
				cmd.Connection.Open();
				cmd.ExecuteNonQuery();
			}
			catch (Exception ex)
			{
				AuditLog("Log MVI Request failed: " + ex.Message, System.Diagnostics.EventLogEntryType.Error);
			}
			finally
			{
				if (cmd.Connection != null)
				{
					if (cmd.Connection.State != ConnectionState.Closed)
					{
						cmd.Connection.Close();
					}
				}
				cmd.Dispose();
				cmd = null;
				rwl.ReleaseWriterLock();
			}

			return rtn;
		}
		public int LogESR(int LogID, float runtime, string status, string payload)
		{
			int rtn = -1;

			rwl.AcquireWriterLock(Timeout.Infinite);

			string SQL = "usp_LogESR";
			SqlConnectionStringBuilder scb = new SqlConnectionStringBuilder();
			scb.PersistSecurityInfo = false;
			scb.IntegratedSecurity = false;
			scb.DataSource = sLogServer;
			scb.InitialCatalog = sLogDatabase;
			scb.UserID = sLogUser;
			scb.Password = sLogPassword;

			SqlCommand cmd = new SqlCommand(SQL, new SqlConnection(scb.ConnectionString));
			cmd.CommandType = CommandType.StoredProcedure;
			try
			{

				cmd.Parameters.AddWithValue("@k_pk_id", LogID);
				cmd.Parameters.AddWithValue("@ESRTime", runtime);
				cmd.Parameters.AddWithValue("@ESRStatus", status);
				cmd.Parameters.AddWithValue("@ESRPayloadData", payload);
				cmd.Connection.Open();
				cmd.ExecuteNonQuery();
			}
			catch (Exception ex)
			{
				AuditLog("Log ESR Request failed: " + ex.Message, System.Diagnostics.EventLogEntryType.Error);
			}
			finally
			{
				if (cmd.Connection != null)
				{
					if (cmd.Connection.State != ConnectionState.Closed)
					{
						cmd.Connection.Close();
					}
				}
				cmd.Dispose();
				cmd = null;
				rwl.ReleaseWriterLock();
			}

			return rtn;
		}

		public void LogResponse(int LogID, string RequestQualifier, string ResponseQualifier, string StatusIndicator, int ErrorID, string ErrorMessage, string ResponsePayload)
		{
			rwl.AcquireWriterLock(Timeout.Infinite);
			string SQL = "usp_LogResponse";

			SqlConnectionStringBuilder scb = new SqlConnectionStringBuilder();
			scb.PersistSecurityInfo = false;
			scb.IntegratedSecurity = false;
			scb.DataSource = sLogServer;
			scb.InitialCatalog = sLogDatabase;
			scb.UserID = sLogUser;
			scb.Password = sLogPassword;

			SqlCommand cmd = new SqlCommand(SQL, new SqlConnection(scb.ConnectionString));
			cmd.CommandType = CommandType.StoredProcedure;
			try
			{
				cmd.Parameters.AddWithValue("@k_pk_id", LogID);
				cmd.Parameters.AddWithValue("@RequestQualifier", RequestQualifier);
				cmd.Parameters.AddWithValue("@ResponseTime", DateTime.Now);
				cmd.Parameters.AddWithValue("@ResponseQualifier", ResponseQualifier);
				cmd.Parameters.AddWithValue("@StatusIndicator", StatusIndicator);
				cmd.Parameters.AddWithValue("@ErrorID", ErrorID);
				cmd.Parameters.AddWithValue("@ErrorMessage", ErrorMessage);
				cmd.Parameters.AddWithValue("@OutboundPayload", ResponsePayload);

				cmd.Connection.Open();
				cmd.ExecuteNonQuery();
			}
			catch (Exception ex)
			{
				AuditLog("Log Response for " + LogID + " failed: " + ex.Message, System.Diagnostics.EventLogEntryType.Error);
			}
			finally
			{
				if (cmd.Connection != null)
				{
					if (cmd.Connection.State != ConnectionState.Closed)
					{
						cmd.Connection.Close();
					}
				}
				cmd.Dispose();
				cmd = null;
				rwl.ReleaseWriterLock();
			}
		}

		//ZEB VA118-11-D-1009 1/23/13 (+method) added to correctly handle STCs in 271 responses
		/// <summary>
		/// Connects to the SQL Server database hosting STC data and recreates the STC table locally
		/// </summary>
		/// <returns>True if DB download was successful, false otherwise</returns>
		public bool LoadSTCData()
		{
			bool result = false;
			SqlConnectionStringBuilder scb = new SqlConnectionStringBuilder();
			//SqlCommand cmd;
			StringBuilder genericCodes = new StringBuilder();

			scb.PersistSecurityInfo = false;
			scb.IntegratedSecurity = false;
			scb.DataSource = sSTCServer;
			scb.InitialCatalog = sSTCDatabase;
			scb.UserID = sSTCUser;
			scb.Password = sSTCPassword;

			//cmd = new SqlCommand("SELECT * FROM Service_Type_Code ORDER BY Service_Type_Code,Service_Description", new SqlConnection(scb.ConnectionString));
			//cmd.CommandType = CommandType.Text;

			try
			{
				using (SqlConnection mySqlConnection = new SqlConnection(scb.ConnectionString))
				{
					using (SqlCommand cmd = new SqlCommand("SELECT * FROM Service_Type_Code ORDER BY Service_Type_Code,Service_Description", mySqlConnection))
					{
						cmd.CommandType = CommandType.Text;
						mySqlConnection.Open();

						using (System.Data.SqlClient.SqlDataReader tableEntries = cmd.ExecuteReader())  //execute the query, allows us to loop over each row in the results
						{
							if (tableEntries.HasRows)
							{
								while (tableEntries.Read())  //loop over each row
								{
									string code = "";
									ServiceTypeInfo info = new ServiceTypeInfo();
									string deductible = tableEntries["Deductible_Explicit"].ToString();
									string copay = tableEntries["Copay_Explicit"].ToString();
									string coinsurance = tableEntries["Coinsurance_Explicit"].ToString();
									bool sendGeneric = tableEntries["Send_Generic_Response"].ToString() == "Y";

									if (deductible == "") deductible = tableEntries["Deductible_Generic"].ToString();
									if (copay == "") copay = tableEntries["Copay_Generic"].ToString();
									if (coinsurance == "") coinsurance = tableEntries["Coinsurance_Generic"].ToString();

									info.Deductible = deductible;
									info.Copay = copay;
									info.Coinsurance = coinsurance;
									info.SendGenericResponse = sendGeneric;

									code = tableEntries["Service_Type_Code"].ToString();
									ServiceTypeCodes.Add(code, info);

									if (tableEntries["Send_Generic_Response"].ToString() == "Y")
									{
										GenericServiceTypes.Add(code);
									}

									if (tableEntries["In_Generic_Response"].ToString() == "Y")
									{
										if (genericCodes.Length == 0)  //build ^-delimited list of generic codes to use when building EB segments
										{
											genericCodes.Append(code);
										}
										else
										{
											genericCodes.Append("^");
											genericCodes.Append(code);
										}
									}
								}

								GenericServiceTypeList = genericCodes.ToString();  //only need to generate this list once

								result = true;
							}
						}
					}
				}
			}
			catch (SqlException ex)
			{
				AuditLog("Load of STC data failed, SQL Exception: " + ex.Message, System.Diagnostics.EventLogEntryType.Error);
				//Logging.WriteLogFile("Load of STC data failed, SQL Exception: " + ex.Message + "\r\n" + ex.StackTrace);
			}
			catch (Exception ex)
			{
				AuditLog("Load of STC data failed, Exception: " + ex.Message, System.Diagnostics.EventLogEntryType.Error);
				//Logging.WriteLogFile("Load of STC data failed, Exception: " + ex.Message + "\r\n" + ex.StackTrace);
			}
			finally
			{
				//if (cmd.Connection != null)
				//{
				//    if (cmd.Connection.State != ConnectionState.Closed)
				//    {
				//        cmd.Connection.Close();
				//    }
				//}
				//cmd.Dispose();
			}
			return result;
		}

		//ZEB VA118-11-D-1009 1/23/13 (+method) added to correctly handle STCs in 271 responses
		/// <summary>
		/// Generates one or more EB segments for a 271 response based on the service codes from the 270 request
		/// </summary>
		/// <param name="ServiceTypes">The ^-delimited list of service type codes from the 270 request</param>
		/// <param name="EligibilityOrBenefitInformationCode">The eligibility code determined from the ES</param>
		/// <returns>The EB segment(s) to send in the 271 response</returns>
		public List<EBSegment> GenerateEBSegments(string ServiceTypes, string EligibilityOrBenefitInformationCode)
		{
			List<EBSegment> result = new List<EBSegment>();

			if (!ServiceTypeCodes.ContainsKey("30")) return result;  //handle case where STC data didn't load correctly

			const string PLAN_NAME = "VA FEE BASIS PROGRAM";

			bool sendGeneric = false;
			string[] codes;

			codes = ServiceTypes.Split('^');  //get each code in the string that came in on the 270

			foreach (string code in codes)
			{
				if (GenericServiceTypes.Contains(code))
				{
					sendGeneric = true;  //handle generic responses at the end so we only return one no matter how many triggers we receive
				}
				else
				{
					if (ServiceTypeCodes.ContainsKey(code))  //if the code is valid (and doesn't call for a generic response), add EB segments for it
					{
						result.Add(new EBSegment(EligibilityOrBenefitInformationCode, "", code, PLAN_NAME, "", "", "", "Y"));
						if (EligibilityOrBenefitInformationCode == "1")  //ZEB VA118-11-D-1009 6/5/13 specifically check for active coverage
						{
							result.Add(new EBSegment("A", "IND", code, PLAN_NAME, "29", "", (float.Parse(ServiceTypeCodes[code].Coinsurance) / 100).ToString(), "Y"));
							result.Add(new EBSegment("B", "IND", code, PLAN_NAME, "", ServiceTypeCodes[code].Copay, "", "Y"));
						}
					}
				}
			}

			if (result.Count == 0) sendGeneric = true;  //if no (valid) explicit codes were passed in, then return the generic response

			if (sendGeneric)
			{
				result.Add(new EBSegment(EligibilityOrBenefitInformationCode, "", GenericServiceTypeList, PLAN_NAME, "", "", "", "Y"));
				if (EligibilityOrBenefitInformationCode == "1")  //ZEB VA118-11-D-1009 6/5/13 specifically check for active coverage
				{
					result.Add(new EBSegment("A", "IND", GenericServiceTypeList, PLAN_NAME, "29", "", ServiceTypeCodes["30"].Coinsurance, "Y"));
					result.Add(new EBSegment("B", "IND", GenericServiceTypeList, PLAN_NAME, "", ServiceTypeCodes["30"].Copay, "", "Y"));
				}
			}

			if (EligibilityOrBenefitInformationCode == "1")  //ZEB VA118-11-D-1009 6/5/13 specifically check for active coverage
			{
				result.Add(new EBSegment("C", "IND", "30", PLAN_NAME, "32", ServiceTypeCodes["30"].Deductible, "", "Y"));  //required base deductible information
				result.Add(new EBSegment("C", "IND", "30", PLAN_NAME, "29", ServiceTypeCodes["30"].Deductible, "", "Y"));
			}

			return result;
		}

	}
}
