﻿/*
 * Program Name: New Standards and Operating Rules - Payer Side (POR)
 * Contractor Name: Harris Corporation
 * Contract Number: VA118-11-D-1009
 * Project: POR - CORE Compliance - Automated Eligibility Tool (AET)
 * 
 * Developer: Zach Bertram
 * Date: 02.19.2013
 * ----------------
 * Created to encapsulate information needed to produce EB segments in the 271 response
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace AETShared
{
    public class EBSegment
    {
        /// <summary>
        /// Field EB01; A code that specifies whether this segment will specify if coverage is active or not, or whether it will describe benefit info like copay or deductible
        /// </summary>
        public string EligibilityOrBenefitInformationCode { get; set; }

        /// <summary>
        /// Field EB02; a code that specifies who is covered -- for Fee, this is always IND-Individual
        /// </summary>
        public string CoverageLevelCode { get; set; }

        /// <summary>
        /// Field EB03; a ^-delimited list of codes for covered service types
        /// </summary>
        public string ServiceTypeCode { get; set; }

        /// <summary>
        /// Field EB05; the name of the benefit plan
        /// </summary>
        public string PlanName { get; set; }

        /// <summary>
        /// Field EB06; a code specifiying the time period for a deductible -- for Fee, this is always 32-Lifetime
        /// </summary>
        public string TimePeriodQualifier { get; set; }

        /// <summary>
        /// Field EB07; the amount of the deductible or copay
        /// </summary>
        public string MonetaryAmount { get; set; }

        /// <summary>
        /// Field EB08; The coinsurance percentage as a decimal -- so 0.5 for 50%, or 1 for 100%.
        /// </summary>
        public string BenefitPercent { get; set; }

        /// <summary>
        /// Field EB11; whether or not an auth/cert is needed for care -- for Fee, this is always Y-Yes
        /// </summary>
        public string AuthCertIndicator { get; set; }

        /// <summary>
        /// Builds a new EB segment that can be filled in
        /// </summary>
        public EBSegment()
        {
        }

        /// <summary>
        /// Builds a new EB segment with the specified values
        /// </summary>
        /// <param name="EligibilityOrBenefitInformationCode">Value for EB01</param>
        /// <param name="CoverageLevelCode">Value for EB02</param>
        /// <param name="ServiceTypeCode">Value for EB03</param>
        /// <param name="PlanName">Value for EB05</param>
        /// <param name="TimePeriodQualifier">Value for EB06</param>
        /// <param name="MonetaryAmount">Value for EB07</param>
        /// <param name="BenefitPercent">Value for EB08</param>
        /// <param name="AuthCertIndicator">Value for EB11</param>
        public EBSegment(string EligibilityOrBenefitInformationCode, string CoverageLevelCode, string ServiceTypeCode, string PlanName, string TimePeriodQualifier, string MonetaryAmount, string BenefitPercent, string AuthCertIndicator)
        {
            this.EligibilityOrBenefitInformationCode = EligibilityOrBenefitInformationCode;
            this.CoverageLevelCode = CoverageLevelCode;
            this.ServiceTypeCode = ServiceTypeCode;
            this.PlanName = PlanName;
            this.TimePeriodQualifier = TimePeriodQualifier;
            this.MonetaryAmount = MonetaryAmount;
            this.BenefitPercent = BenefitPercent;
            this.AuthCertIndicator = AuthCertIndicator;
        }

        /// <summary>
        /// Outputs the text of the EB segment as it would appear in the 271 message
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return String.Format("EB*{0}*{1}*{2}**{3}*{4}*{5}*{6}***{7}~", EligibilityOrBenefitInformationCode, CoverageLevelCode, ServiceTypeCode, PlanName, TimePeriodQualifier, MonetaryAmount, BenefitPercent, AuthCertIndicator);
        }
    }
}
