﻿using System;
using System.Globalization;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using X12Engine.BLL.X12._278;
using System.Data;
using System.IO;

namespace X12Engine
{
	public static class Utils
	{

		public static DateTime? ParseDate(string element, string dateFormat)
		{
			if (string.IsNullOrEmpty(element)) return null;

			DateTime dateValue = DateTime.MinValue;

			if (string.IsNullOrEmpty(dateFormat))
			{
				dateFormat = "yyyyMMdd";
			}

			int dateFormatLength = (element.Length < dateFormat.Length) ? element.Length : dateFormat.Length;

			if (DateTime.TryParseExact(element.Substring(0, dateFormatLength), dateFormat, new CultureInfo("en-US"), DateTimeStyles.None, out dateValue))
				return dateValue;
			else
				return null;
		}

		public static DateTime? ParseTime(string element, string dateFormat, DateTime? date)
		{
			if (string.IsNullOrEmpty(element)) return null;

			DateTime dateValue = DateTime.MinValue;

			if (string.IsNullOrEmpty(dateFormat))
			{
				if (element.Length == 4) { dateFormat = "HHmm"; }
				if (element.Length == 5) { dateFormat = "HHmms"; }
				if (element.Length == 6) { dateFormat = "HHmmss"; }
				if (element.Length == 7) { dateFormat = "HHmmssf"; }
				if (element.Length == 8) { dateFormat = "HHmmssff"; }
			}

			int dateFormatLength = (element.Length < dateFormat.Length) ? element.Length : dateFormat.Length;

			if (DateTime.TryParseExact(element.Substring(0, dateFormatLength), dateFormat, new CultureInfo("en-US"), DateTimeStyles.NoCurrentDateDefault, out dateValue))
			{
				//if the date parameter is included, add the date portion to the time value
				if (date != null)
				{
					DateTime dateTemp = new DateTime(date.Value.Year, date.Value.Month, date.Value.Day, dateValue.Hour, dateValue.Minute, dateValue.Second, dateValue.Millisecond);
					dateValue = dateTemp;
				}

				return dateValue;
			}
			else
				return null;
		}

		public static decimal ParseDecimal(string element)
		{
			decimal d = 0;

			if (decimal.TryParse(element, out d))
				return d;
			else
				return 0;
		}

		public static int ParseInt(string element)
		{
			int i = 0;

			if (int.TryParse(element, out i))
				return i;
			else
				return 0;
		}

		public static string TerminateSegment(Settings settings, string segment)
		{
			while (segment.Substring(segment.Length - 1, 1) == settings.Data_Element_Separator.ToString())
				segment = segment.Substring(0, segment.Length - 1);
			if (settings.Segment_Terminator.ToString() == "^")
			{
				return segment + settings.Segment_Terminator + "\r\n";
			}
			else
			{
				return segment + settings.Segment_Terminator;
			}

		}

		public static string TerminateComponent(Settings settings, string component)
		{
			if (component.Length == 0 || component == "::") return component;
			while (component.Substring(component.Length - 1, 1) == settings.Component_Element_Separator.ToString() && component.Length > 1)
			{
				component = component.Substring(0, component.Length - 1);

			}


			return component;
		}

		public static bool IsEntitySegment(string segmentId)
		{
			switch (segmentId)
			{
				case "NM1":
				case "REF":
				case "N2":
				case "N3":
				case "N4":
				case "PER":
				case "AAA":
				case "PRV":
				case "DMG":
				case "INS":
				case "DTP":
					return true;
				default:
					return false;
			}
		}

		public static bool IsContactEntity(string code)
		{
			switch (code)
			{
				case "00":
				case "CA":
				case "GG":
					return true;
				default:
					return false;
			}
		}



		public static bool IsProviderEntity(string code)
		{
			switch (code)
			{
				case "SJ":
				case "71":
				case "72":
				case "73":
				case "77":
				case "AAJ":
				case "DD":
				case "DK":
				case "DN":
				case "FA":
				case "G3":
				case "P3":
				case "QB":
				case "QV":
				
					return true;
				default:
					return false;
			}
		}



		public static bool IsTransportEntity(string code)
		{
			switch (code)
			{
				case "45":
				case "FS":
				case "ND":
				case "PW":
				case "R3":
					return true;
				default:
					return false;
			}
		}

		public static string[] Split(string input, string key)
		{
			string[] output = null;

			int index = input.IndexOf(key);

			while (index > 0)
			{
				string temp = input.Substring(0, index);
				AddToArray(ref output, temp);
				input = input.Remove(0, index);
				index = input.IndexOf(key, key.Length);
			}

			AddToArray(ref output, input);

			return output;
		}

		public static void AddToArray<t>(ref t[] arrObject, t obj)
		{
			if (arrObject == null)
			{
				arrObject = (t[])Array.CreateInstance(typeof(t), 1);
			}
			else
			{
				Array.Resize(ref arrObject, arrObject.Length + 1);
			}

			arrObject.SetValue(obj, arrObject.GetUpperBound(0));
		}

		public static bool Contains(string[] arrObject, string obj)
		{
			if (arrObject != null)
			{
				foreach (string s in arrObject)
				{
					if (s == obj) return true;
				}
			}

			return false;
		}

		public static bool Contains(string[] arrObject, string obj, bool ignoreCase)
		{
			if (arrObject != null)
			{
				foreach (string s in arrObject)
				{
					if (ignoreCase == true && s.ToUpper() == obj.ToUpper()) return true;
					else if (s == obj) return true;
				}
			}

			return false;
		}


		public static int CountSegments(Settings settings, string message)
		{
			int segmentCount = 0;

			if (!string.IsNullOrEmpty(message))
			{
				char[] chars = message.ToCharArray();
				if (chars != null)
				{
					foreach (char c in chars)
					{
						if (c == settings.Segment_Terminator) segmentCount++;
					}
				}
			}

			return segmentCount;
		}

		public static int GetIntValue(object obj)
		{
			try
			{
				if (obj != null)
				{
					return Convert.ToInt32(obj);
				}
				return 0;
			}
			catch (Exception)
			{
				return 0;
			}

		}
		public static long GetLongValue(object obj)
		{
			try
			{
				if (obj != null)
				{
					return Convert.ToInt64(obj);
				}
				return 0;
			}
			catch (Exception)
			{
				return 0;
			}

		}
		public static string GetStringValue(object obj)
		{
			if (obj != null)
			{
				return obj.ToString();
			}
			return "";
		}

		public static DateTime? GetDateValue(object obj)
		{
			try
			{
				DateTime? nullDate = null;
				if (obj != null)
				{
					return Convert.ToDateTime(obj);
				}
				return nullDate;
			}
			catch (Exception)
			{
				return null;
			}

		}


		public static void LogEvent(string logContent)
		{

			try
			{

				//TimeSpan timeSpan = new TimeSpan();
				//timeSpan = DateTime.Now - AAAManager.ProcessDateTime;

				string LogPath = "C:\\Program Files\\VistAFee5010\\AETService\\Logs\\AETLog_" + DateTime.Now.Year + DateTime.Now.Month + DateTime.Now.Day + ".txt";

				if (File.Exists(LogPath))
				{
					using (StreamWriter outfile = File.AppendText(LogPath))
					{
						outfile.WriteLine("\r\n-----------------------------------------------------------------------");
						outfile.WriteLine(DateTime.Now);
						outfile.WriteLine(logContent);
					}
				}
				else
				{
					using (StreamWriter outfile = new StreamWriter(LogPath))
					{
						outfile.WriteLine("\r\n-----------------------------------------------------------------------");
						outfile.WriteLine(DateTime.Now);
						outfile.WriteLine(logContent );
					}
				}

				//AAAManager.ProcessDateTime = DateTime.Now;
			}
			catch (Exception)
			{

			}
		}

		public static void ClearLog()
		{
			//using (SqlConnection conn = new SqlConnection(Properties.Settings.Default.AATConnection))
			//{
			//   conn.Open();

			//   using (SqlCommand cmd = new SqlCommand("DELETE FROM EventLog", conn))
			//   {
			//      cmd.ExecuteNonQuery();
			//   }
			//}
		}

		public static string GetLog()
		{
			StringBuilder sb = new StringBuilder();
			//using (SqlConnection conn = new SqlConnection(Properties.Settings.Default.AATConnection))
			//{
			//   conn.Open();

			//   using (SqlCommand cmd = new SqlCommand("SELECT Entered, LogContent FROM EventLog ORDER BY ID", conn))
			//   {
			//      using (SqlDataReader reader = cmd.ExecuteReader())
			//      {
			//         while (reader.Read())
			//         {
			//            sb.AppendLine(reader["Entered"].ToString() + " - " + reader["LogContent"].ToString());
			//         }
			//      }
			//   }
			//}
			return sb.ToString();
		}




	}
}

