package va.gov.util;

import java.net.URL;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.SessionFactory;



/**
 *  This class is an implementation of the Service Locator pattern. It is 
 *  used to looukup resources such as EJBHomes, JMS Destinations, etc. 
 */
public class ServiceLocator {

    private InitialContext ic;
    
    private static final Logger LOGGER = Logger.getLogger(ServiceLocator.class);
  
    

    public ServiceLocator() throws ServiceLocatorException  {
      try {
        ic = new InitialContext();
      } catch (NamingException ne) {
    	  if (LOGGER.isDebugEnabled()){
    		  LOGGER.error("Exception in ServiceLocator:ServiceLocator " + ne.getMessage() );
			}
      } catch (Exception e) {
    	  if (LOGGER.isDebugEnabled()){
    		  LOGGER.error("Exception in ServiceLocator:ServiceLocator " + e.getMessage() );
			}
      }
    }

    /**
     * Gets a <code>DataSource</code> using the given JNDI name.
     *
     * @param dataSourceJndiName The <code>DataSource</code>'s JNDI name.
     *
     * @return DataSource The <code>DataSource</code>.
     *
     * @throws ServiceLocatorException If there are JNDI lookup problems.
     *
     * @see javax.sql.DataSource
     */
    public static DataSource getDataSource(String dataSourceJndiName) throws ServiceLocatorException {
        DataSource dataSource = null;

        try {
            Context ctx = new InitialContext();
            Context envCtx = (Context) ctx.lookup("java:comp/env");
            dataSource = (DataSource) envCtx.lookup(dataSourceJndiName);


         //   dataSource = (DataSource) ctx.lookup(dataSourceJndiName);

        } catch (ClassCastException cce) {
        	if (LOGGER.isDebugEnabled()){
        		LOGGER.error("Exception in ServiceLocator:getDataSource " + cce.getMessage() );
			}
        } catch (NamingException ne) {
        	if (LOGGER.isDebugEnabled()){
        		LOGGER.error("Exception in ServiceLocator:getDataSource " + ne.getMessage() );
			}
        }

        return dataSource;
    }
/**
 * getHibernateSessionFactory used to create Session  for 
 * communication with persistent layer.
 * @param jndiSessionFactoryName
 * @return SessionFactory
 * @throws ServiceLocatorException
 */
    public static SessionFactory getHibernateSessionFactory(String jndiSessionFactoryName) throws ServiceLocatorException {
        SessionFactory sessionFactory = null;

        try {
            Context ctx = new InitialContext();
            Context envCtx = (Context) ctx.lookup("java:comp/env");
            sessionFactory = (SessionFactory) envCtx.lookup(jndiSessionFactoryName);
            
        } catch (ClassCastException cce) {
        	if (LOGGER.isDebugEnabled()){
        		LOGGER.error("Exception in ServiceLocator:getHibernateSessionFactory " + cce.getMessage() );
			}
        } catch (NamingException ne) {
        	if (LOGGER.isDebugEnabled()){
        		LOGGER.error("Exception in ServiceLocator:getHibernateSessionFactory " + ne.getMessage() );
			}
        }

        return sessionFactory;
    }
    /**
     * getHibernateSession used for create session for data transmission.
     * @param jndiSessionFactoryName
     * @return Session
     * @throws ServiceLocatorException
     */
    public static Session getHibernateSession(String jndiSessionFactoryName) throws ServiceLocatorException {
        Session hibernateSess = null;

        try 
        {
            Context ctx = new InitialContext();
            Context envCtx = (Context) ctx.lookup("java:comp/env");
            
            SessionFactory sessionFactory = (SessionFactory)envCtx.lookup(jndiSessionFactoryName);
            hibernateSess = sessionFactory.getCurrentSession();

            
          //  session = getHibernateSessionFactory(jndiSessionFactoryName).openSession();            
        } 
        catch (Exception e) 
        {
        	if (LOGGER.isDebugEnabled()){
        		LOGGER.error("Exception in ServiceLocator:getHibernateSession " + e.getMessage() );
			}
        }
        return hibernateSess;
    }
    
    
  

    /**
     * 
     * @return the URL value corresponding
     * to the env entry name.
     */
    public URL getUrl(String envName) throws ServiceLocatorException {  
      URL url = null;
      try {
        url = (URL)ic.lookup(envName);   
      } catch (NamingException ne) {
    	  if (LOGGER.isDebugEnabled()){
    		  LOGGER.error("Exception in ServiceLocator:getUrl " + ne.getMessage() );
			}
      } catch (Exception e) {
    	  if (LOGGER.isDebugEnabled()){
    		  LOGGER.error("Exception in ServiceLocator:getUrl " + e.getMessage() );
			}
      }

      return url;         
    }

    /**
     * @return the boolean value corresponding
     * to the env entry such as SEND_CONFIRMATION_MAIL property.
     */
    public boolean getBoolean(String envName) throws ServiceLocatorException {   
      Boolean bool = null;
      try {    
        bool = (Boolean)ic.lookup(envName);          
      } catch (Exception e) {
    	  if (LOGGER.isDebugEnabled()){
    		  LOGGER.error("Exception in ServiceLocator:getBoolean " + e.getMessage() );
			}
      }
      if (bool!=null)
      return bool.booleanValue();
      else 
    	  return false;
    			 
    }

    /**
     * @return the String value corresponding
     * to the env entry name.
     */
    public String getString(String envName) throws ServiceLocatorException { 
      String envEntry = null;   
      try {  
        envEntry = (String)ic.lookup(envName);      
      } catch (NamingException ne) {
    	  if (LOGGER.isDebugEnabled()){
    		  LOGGER.error("Exception in ServiceLocator:getString " + ne.getMessage() );
			}
      } catch (Exception e) {
    	  if (LOGGER.isDebugEnabled()){
    		  LOGGER.error("Exception in ServiceLocator:getString " + e.getMessage() );
			}
      }
      return envEntry ;
    }
    
}


